import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*; 
import java.lang.reflect.*;

		
public class GraphicEditor extends JFrame
{
		
    ToolsBar tb = new ToolsBar();
    GraphicsArea ga = new GraphicsArea();
    StatusBar sb = new StatusBar();


    // *** ToolsBar

    class ToolsBar extends JPanel
    {
	JComboBox c1 = new JComboBox();
	String[] colorNames = {"orange","pink","cyan","magenta","yellow","black","white","gray","lightGray","darkGray","red","green","blue"};
	
	JComboBox c2 = new JComboBox();
	String[] shapeNames = {"Line", "Oval", "Rectangle"};
	
	JCheckBox cb1 = new JCheckBox("filled");
	JCheckBox cb2 = new JCheckBox("starry");	
	
	JButton b = new JButton("Clear");
		
	public ToolsBar()
	{
	    b.addActionListener(new ActionListener() 
		{
		    public void actionPerformed(ActionEvent e)
		    {
			ga.repaint();
		    }
		});
	    

	    for(int i=0; i < colorNames.length; i++) c1.addItem(colorNames[i]);
	    for(int i=0; i < shapeNames.length; i++) c2.addItem(shapeNames[i]);
	    
	    setBackground(Color.lightGray);
	    setLayout(new FlowLayout(FlowLayout.CENTER,25,0));
	    add(c1); add(c2); add(cb1); add(cb2); add(b);
	}

	public Color getSelectedColor()
	{
	    Color color=null;
	    try
	    {
		Class cla=Color.class; 
		Field fd = cla.getField((String)(c1.getSelectedItem())); 
		color = (Color) fd.get(null); 
	    }
	    catch (Exception e) { System.out.println("pb"); }
	    return color;
	}

	public String getSelectedShapeName() { return (String)(c2.getSelectedItem()); }
	public boolean getFilled() { return cb1.isSelected(); } 
	public boolean getStarry() { return cb2.isSelected(); } 

    } // class toolsbar


	
    // *** GarphicsArea
    
    class GraphicsArea extends JPanel
    {
	private Graphics g;
	private Color bgColor=Color.white;
	private int x1, y1;
	
	public GraphicsArea()
	{
	    setBackground(bgColor);
	    setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
	}

	
	public void memorizeOrigin(int x, int y)
	{
	    x1=x; y1=y;
	    sb.displayInfo("Release to draw the shape");
	}

	public void drawShape(int x2, int y2)
	{
	    if (g == null) g=getGraphics();
	    g.setColor(tb.getSelectedColor());
	    if (tb.getSelectedShapeName().compareTo("Line") == 0)
	    {
		g.drawLine(x1,y1,x2,y2);
	    }
	    else if (tb.getSelectedShapeName().compareTo("Oval") == 0)
	    {
		if (tb.getFilled()) 
		    g.fillOval(Math.min(x1,x2),Math.min(y1,y2),
			       Math.abs(x2-x1),Math.abs(y2-y1));
		else 
		    g.drawOval(Math.min(x1,x2),Math.min(y1,y2),
			       Math.abs(x2-x1),Math.abs(y2-y1));
	    }
	    else if (tb.getSelectedShapeName().compareTo("Rectangle") == 0)
	    {
		if (tb.getFilled()) 
		    g.fillRect(Math.min(x1,x2),Math.min(y1,y2),
			       Math.abs(x2-x1),Math.abs(y2-y1));
		else 
		    g.drawRect(Math.min(x1,x2),Math.min(y1,y2),
			       Math.abs(x2-x1),Math.abs(y2-y1));
	    }
	    sb.displayInfo("Click to initiate a shape");
	}


	public void move(int x2, int y2)
	{
	    if (tb.getStarry()) drawShape(x2,y2);
	}

    } // class GraphicsArea


	
    class StatusBar extends JPanel
    {
	JLabel info = new JLabel();
	JLabel coord = new JLabel();
	
	public StatusBar()
	{
	    setBackground(Color.lightGray);
	    setLayout(new GridLayout(1,2));
	    add(info); add(coord); 
	}
	
	public void displayInfo(String message)
	{
	    info.setText(message);
	}
	
	public void displayCoord(int x, int y)
	{
	    coord.setText("x : " + x + "   y : " +y);
	}
    }
    
    
    // *** constructor
    
    public GraphicEditor()
    {
	addWindowListener(new WC());
	ga.addMouseMotionListener(new gaMouseMotionListener());
	ga.addMouseListener(new gaMouseListener());
	
	setTitle("Graphic Editor");
	setBackground(Color.lightGray);
	getContentPane().setLayout(new BorderLayout(5,5));
	getContentPane().add(tb,BorderLayout.NORTH);
	getContentPane().add(ga,BorderLayout.CENTER);
	getContentPane().add(sb,BorderLayout.SOUTH);
	
	setSize(750,450);
	setLocation(10,10);
	setVisible(true);
    }
    
    
    // classes listener et adpater
    
    class WC extends WindowAdapter
    {
	public void windowClosing(WindowEvent e) { System.exit(0); }
    }	
    
    
    class gaMouseMotionListener implements MouseMotionListener
    {
	public void mouseMoved(MouseEvent e)
	{
	    sb.displayCoord(e.getX(), e.getY());
	}
	
	public void mouseDragged(MouseEvent e)
	{
	    ga.move(e.getX(), e.getY());
	    sb.displayCoord(e.getX(), e.getY());
	}
    }	
    
    
    class gaMouseListener extends MouseAdapter
    {
	public void mousePressed(MouseEvent e)
	{
	    ga.memorizeOrigin(e.getX(), e.getY());
	}
	
	public void mouseReleased(MouseEvent e)
	{
	    ga.drawShape(e.getX(), e.getY());
	}
    }	
    
    
    public static void main(String[] args) { new GraphicEditor(); }

}
